from __future__ import annotations

import sys
from typing import TYPE_CHECKING

from django.apps import apps as django_apps
from django.core.management.color import color_style
from django.utils.text import slugify

from .constants import SEARCH_SLUG_SEP

style = color_style()
if TYPE_CHECKING:
    from edc_model.models import BaseUuidModel

    from .model_mixins import SearchSlugModelMixin

    class Model(SearchSlugModelMixin, BaseUuidModel):
        pass


class SearchSlug:
    def __init__(self, obj: Model | SearchSlugModelMixin = None, fields: list[str] = None):
        self.warning = None
        self.slug = ""
        self.model_cls = None
        self.fields = None
        if obj and fields:
            self.model_cls = django_apps.get_model(obj._meta.label_lower)
            self.fields = self.get_safe_fields(fields)
            values = []
            for field in fields:
                value = obj
                for f in field.split("."):
                    value = getattr(value, f)
                values.append(value)
            slugs = [slugify(item or "") for item in values]
            slug = SEARCH_SLUG_SEP.join(slugs)
            if len(slug) > 250:
                self.warning = (
                    f"Warning! Search slug string exceeds 250 chars. See {repr(obj)}\n"
                )
                sys.stdout.write(style.WARNING(self.warning))
            self.slug = slug[:250]

    def get_safe_fields(self, fields):
        from django_crypto_fields.fields import BaseField

        encrypted_fields = [
            fld.name for fld in self.model_cls._meta.fields if isinstance(fld, BaseField)
        ]
        return [f for f in fields if f not in encrypted_fields]



#!/usr/bin/env python
import sys
from pathlib import Path

from edc_test_settings.default_test_settings import DefaultTestSettings

app_name = "edc_search"
base_dir = Path(__file__).absolute().parent.parent.parent

project_settings = DefaultTestSettings(
    calling_file=__file__,
    BASE_DIR=base_dir,
    APP_NAME=app_name,
    ETC_DIR=base_dir / app_name / "tests" / "etc",
    SUBJECT_SCREENING_MODEL="edc_metadata.subjectscreening",
    SUBJECT_CONSENT_MODEL="edc_metadata.subjectconsent",
    SUBJECT_VISIT_MODEL="edc_visit_tracking.subjectvisit",
    SUBJECT_VISIT_MISSED_MODEL="edc_metadata.subjectvisitmissed",
    INSTALLED_APPS=[
        "django.contrib.admin",
        "django.contrib.auth",
        "django.contrib.contenttypes",
        "django.contrib.sessions",
        "django.contrib.messages",
        "django.contrib.staticfiles",
        "django.contrib.sites",
        "django_crypto_fields.apps.AppConfig",
        "django_revision.apps.AppConfig",
        "multisite",
        "edc_search.apps.AppConfig",
    ],
    RANDOMIZATION_LIST_PATH=base_dir / app_name / "tests" / "test_randomization_list.csv",
    add_dashboard_middleware=True,
).settings

for k, v in project_settings.items():
    setattr(sys.modules[__name__], k, v)



from __future__ import annotations

from typing import TYPE_CHECKING

from .search_slug import SearchSlug

if TYPE_CHECKING:
    from edc_model.models import BaseUuidModel

    from .model_mixins import SearchSlugModelMixin

    class Model(SearchSlugModelMixin, BaseUuidModel):
        pass


class SearchSlugDuplicateFields(Exception):
    pass


class SearchSlugUpdater:
    search_slug_cls = SearchSlug

    def __init__(self, fields: list[str], model_obj: Model | SearchSlugModelMixin = None):
        if len(fields) > len(list(set(fields))):
            raise SearchSlugDuplicateFields(
                f"Duplicate search slug fields detected. Got {fields}. " f"See {repr(self)}"
            )
        search_slug = self.search_slug_cls(obj=model_obj, fields=fields)
        self.warning = search_slug.warning
        self.slug = search_slug.slug



__all__ = ["TestModel", "TestModelExtra"]

from django.db import models

from ..model_mixins import SearchSlugManager, SearchSlugModelMixin


class Dummy:
    attr = "dummy_attr"

    def __str__(self):
        return "Dummy"


class TestModelMixin(SearchSlugModelMixin, models.Model):
    f1 = models.CharField(max_length=25, null=True)

    f2 = models.DateTimeField(null=True)

    f3 = models.IntegerField(null=True)

    objects = SearchSlugManager()

    @property
    def attr(self):
        return "attr"

    @property
    def dummy(self):
        return Dummy()

    def get_search_slug_fields(self):
        fields = super().get_search_slug_fields()
        fields.extend(["f1", "f2", "f3", "attr", "dummy", "dummy.attr"])
        return fields

    class Meta:
        abstract = True


class TestModel(TestModelMixin, models.Model):
    pass


class TestModelExtra(TestModelMixin, models.Model):
    f4 = models.CharField(max_length=25, null=True)

    def get_search_slug_fields(self):
        fields = super().get_search_slug_fields()
        fields.append("f4")
        return fields


class TestModelDuplicate(TestModelMixin, models.Model):
    f4 = models.CharField(max_length=25, null=True)

    def get_search_slug_fields(self):
        fields = super().get_search_slug_fields()
        fields.extend(["f1", "f4"])
        return fields
